/**
 * empty function
 * @alias noop
 * @export
 */
function noop() { }

/**
 * @export
 * @class EventEmitter
 * @alias event/EventEmitter
 * @memberof event
 */
class EventEmitter {
    constructor() {
        this.events = [];
    }
    /**
     * 이벤트 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    on(eventName, listener = noop, context) {
        this.events.push({ eventName, listener, context });
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    off(eventName, listener) {
        // tslint:disable-next-line: max-line-length
        const matched = this.events.findIndex(regEvent => regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true));
        // tslint:disable-next-line: no-bitwise
        if (!~matched) {
            return;
        }
        this.events.splice(matched, 1);
        return this;
    }
    /**
     * 이벤트 1회 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    once(eventName, listener = noop, context) {
        const onceWrapper = (...values) => {
            this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    emit(eventName, ...values) {
        this.events.forEach(regEvent => {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    fire(eventName, ...values) {
        this.emit(eventName, values);
        return this;
    }
}

/**
 * @alias event
 */

/*
 * Public API Surface of @nonoll/code-snippet/event
 */

/**
 * Generated bundle index. Do not edit.
 */

export { EventEmitter };
//# sourceMappingURL=nonoll-code-snippet-event.js.map
