/**
 * empty function
 * @alias noop
 * @export
 */
function noop() { }

/**
 * @export
 * @class EventEmitter
 * @alias event/EventEmitter
 * @memberof event
 */
class EventEmitter {
    constructor() {
        this.events = [];
    }
    /**
     * 이벤트 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    on(eventName, listener = noop, context) {
        this.events.push({ eventName, listener, context });
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    off(eventName, listener) {
        // tslint:disable-next-line: max-line-length
        const matched = this.events.findIndex(regEvent => regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true));
        // tslint:disable-next-line: no-bitwise
        if (!~matched) {
            return;
        }
        this.events.splice(matched, 1);
        return this;
    }
    /**
     * 이벤트 1회 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    once(eventName, listener = noop, context) {
        const onceWrapper = (...values) => {
            this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    emit(eventName, ...values) {
        this.events.forEach(regEvent => {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    fire(eventName, ...values) {
        this.emit(eventName, values);
        return this;
    }
}

const debounce = (fn, delay) => {
    let timer;
    delay = delay || 0;
    return (...args) => {
        window.clearTimeout(timer);
        timer = window.setTimeout(() => {
            fn.apply(null, args);
        }, delay);
    };
};

/**
 * @typedef {String} MutationRecordType
 * @alias MutationRecordType
 * @memberof MutationObserver
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @property {String} childList
 * @property {String} attributes
 * @property {String} characterData
 * @property {String} subtree
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MutationRecordType}
 */
var MutationRecordType;
(function (MutationRecordType) {
    MutationRecordType["childList"] = "childList";
    MutationRecordType["attributes"] = "attributes";
    MutationRecordType["characterData"] = "characterData";
    MutationRecordType["subtree"] = "subtree";
})(MutationRecordType || (MutationRecordType = {}));
/**
 * MutationEvent Types
 * @event MutationObserver#MUTATION_EVENTS
 * @memberof MutationObserver
 * @property {String} CHANGE_CHILD_LIST - childList 변경 시점
 * @property {String} CHANGE_SUBTREE - subtree 변경 시점
 * @property {String} CHANGE_ATTRIBUTES - attributes 변경 시점
 * @property {String} CHANGE_CHARACTER_DATA - data 변경 시점
 * @property {String} CHANGE - 기타 변경 시점
 * @property {String} WILD_CARD - 모든 변경 시점
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MUTATION_EVENTS}
 */
var MUTATION_EVENTS;
(function (MUTATION_EVENTS) {
    MUTATION_EVENTS["CHANGE_CHILD_LIST"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHILD_LIST";
    MUTATION_EVENTS["CHANGE_SUBTREE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_SUBTREE";
    MUTATION_EVENTS["CHANGE_ATTRIBUTES"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_ATTRIBUTES";
    MUTATION_EVENTS["CHANGE_CHARACTER_DATA"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHARACTER_DATA";
    MUTATION_EVENTS["CHANGE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE";
    MUTATION_EVENTS["WILD_CARD"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_WILCD_CARD";
})(MUTATION_EVENTS || (MUTATION_EVENTS = {}));
class PureMutationObserver {
    constructor(callback) {
        // TODO polyfill
        // const MutationObserver = require('mutation-observer');
        // this.pure = new MutationObserver(callback);
        this.pure = new window.MutationObserver(callback);
    }
    observe(target, options) {
        this.pure.observe(target, options);
    }
    disconnect() {
        this.pure.disconnect();
    }
}
/**
 * target 으로 설정된 element 의 변화를 감지
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobserver-MutationObserver.html&module=%2Fobserver-MutationObserver.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class MutationObserver
 * @alias observer/MutationObserver
 * @extends EventEmitter
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver
 * @see https://github.com/webmodules/mutation-observer
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @example
import { MutationObserver, MUTATION_EVENTS } from '@nonoll/code-snippet/observer';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

const forExample = () => {
  const doc = window.document;

  const attachButton = createElement({ tag: 'button', text: 'observer attach' });
  const detachButton = createElement({ tag: 'button', text: 'observer detach' });
  const appendButton = createElement({ tag: 'button', text: 'append' });

  doc.body.appendChild(attachButton);
  doc.body.appendChild(detachButton);
  doc.body.appendChild(appendButton);

  attachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('attachButton clicked');
    if (!observer) {
      return;
    }
    observer.attach();
  });

  detachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('detachButton clicked');
    if (!observer) {
      return;
    }
    observer.detach();
  });

  appendButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('appendButton clicked');
    if (!observer) {
      return;
    }
    const input = createElement({ tag: 'input', value: `${+new Date()}` });
    target.appendChild(input);
  });
}

const target = createElement({ id: 'example_target', style: 'border: 1px solid red' });
window.document.body.appendChild(target);

const options = {
  childList: true,
  subtree: true
};

const observer = new MutationObserver({ target, options });

observer.on(MUTATION_EVENTS.WILD_CARD, (type, values) => {
  console.log('wildCard', type, values);
}).on(MUTATION_EVENTS.CHANGE_CHILD_LIST, values => {
  console.log('childList', values);
});

observer.attach();

forExample();
 * @param {Partial<IMutationObserverExtOption>} option
 * @param {HTMLElement} [option.target=null] target
 * @param {Number} [option.debounce=300] debounce
 * @param {Function|null} [option.callback={@link noop}] callback function
 * @param {Object} [option.options={}]
 * @param {Boolean} [option.options.childList]
 * @param {Boolean} [option.options.attributes]
 * @param {Boolean} [option.options.characterData]
 * @param {Boolean} [option.options.subtree]
 * @param {Boolean} [option.options.attributeOldValue]
 * @param {Boolean} [option.options.characterDataOldValue]
 * @param {Array.<String>} [option.options.attributeFilter] [MDN attributeFilter]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationObserverInit/attributeFilter}
 */
class MutationObserver extends EventEmitter {
    constructor({ target = null, debounce: debounce$1 = 300, callback = noop, options = {} }) {
        super();
        const defaultOptions = { childList: true, subtree: true };
        this.target = target;
        this.options = Object.assign({}, defaultOptions, options);
        this.onMutationObserverListener = this.onMutationObserverListener.bind(this);
        this.callback = callback || noop;
        if (debounce$1) {
            this.observer = new PureMutationObserver(debounce(this.onMutationObserverListener, debounce$1));
        }
        else {
            this.observer = new PureMutationObserver(this.onMutationObserverListener);
        }
    }
    /**
     * 이벤트 감지 설정
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.attach();
     */
    attach() {
        this.observer.observe(this.target, this.options);
        return this;
    }
    /**
     * 이벤트 감지 등록
     * @override
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @memberof MutationObserver
     * @listens MutationObserver#MUTATION_EVENTS
     * @returns {MutationObserver}
     */
    on(eventName, listener = noop, context) {
        super.on(eventName, listener, context);
        return this;
    }
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @param {MutationRecord[]} mutations [MDN MutationRecord]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationRecord}
     * @memberof MutationObserver
     * @fires MutationObserver#MUTATION_EVENTS
     */
    onMutationObserverListener(mutations) {
        mutations.forEach(mutation => {
            const type = mutation.type;
            const values = { type, mutation };
            switch (type) {
                case MutationRecordType.childList:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.subtree:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.attributes:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.characterData:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                default:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE, type, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
            }
        });
    }
    /**
     * 이벤트 감지 해제
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.detach();
     */
    detach() {
        if (this.observer) {
            this.observer.disconnect();
        }
        return this;
    }
    /**
     * destory
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.destroy();
     */
    destroy() {
        this.detach();
        this.observer = null;
        return this;
    }
}

const doc = window.document;
function getVendorPrefix() {
    let prefix = '';
    if (typeof doc.hidden !== 'undefined') {
        prefix = '';
    }
    else if (typeof doc.msHidden !== 'undefined') {
        prefix = 'ms';
    }
    else if (typeof doc.webkitHidden !== 'undefined') {
        prefix = 'webkit';
    }
    return prefix;
}
const VENDOR_PREFIX = getVendorPrefix();
const VISIBILITY_EVENT_NAME = `${VENDOR_PREFIX}visibilitychange`;
const HIDDEN_METHOD_NAME = VENDOR_PREFIX ? `${VENDOR_PREFIX}Hidden` : 'hidden';
var VISIBILITY_EVENTS;
(function (VISIBILITY_EVENTS) {
    VISIBILITY_EVENTS["CHANGE"] = "VISIBILITY_CHANGE-EVENTS-CHANGE";
    VISIBILITY_EVENTS["GET_STATUS"] = "VISIBILITY_CHANGE-EVENTS-GET_STATUS";
})(VISIBILITY_EVENTS || (VISIBILITY_EVENTS = {}));
class VisibilityChangeObserver extends EventEmitter {
    constructor() {
        super();
        if (!VisibilityChangeObserver.isSupportVisibilityChange()) {
            throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
        }
        this.onVisibilityChangeListener = this.onVisibilityChangeListener.bind(this);
    }
    static visibilityEventName() {
        return `${VENDOR_PREFIX}visibilitychange`;
    }
    static hiddenMethodName() {
        return VENDOR_PREFIX ? `${VENDOR_PREFIX}Hidden` : 'hidden';
    }
    static isSupportVisibilityChange() {
        return typeof doc[VisibilityChangeObserver.hiddenMethodName()] !== 'undefined';
    }
    onVisibilityChangeListener() {
        const isHidden = this.isHidden();
        this.emit(VISIBILITY_EVENTS.CHANGE, { isHidden, isShow: !isHidden });
    }
    isHidden() {
        return doc[VisibilityChangeObserver.hiddenMethodName()];
    }
    getStatus() {
        this.emit(VISIBILITY_EVENTS.GET_STATUS);
        return this;
    }
    attach() {
        if (!doc.addEventListener) {
            return;
        }
        doc.addEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.on(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    detach() {
        if (!doc.addEventListener) {
            return;
        }
        doc.removeEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.off(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    destroy() {
        this.detach();
        return this;
    }
}

/**
 * @alias observer
 */

/*
 * Public API Surface of @nonoll/code-snippet/observer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { HIDDEN_METHOD_NAME, MUTATION_EVENTS, MutationObserver, MutationRecordType, VENDOR_PREFIX, VISIBILITY_EVENTS, VISIBILITY_EVENT_NAME, VisibilityChangeObserver, getVendorPrefix };
//# sourceMappingURL=nonoll-code-snippet-observer.js.map
